# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: plugin.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cplugin.proto\x12\x06plugin\"R\n\x16\x45ndpointRequestMessage\x12\x0e\n\x06method\x18\x01 \x01(\t\x12(\n\x07request\x18\x02 \x01(\x0b\x32\x17.plugin.EndpointRequest\"`\n\x17\x45ndpointResponseMessage\x12\n\n\x02id\x18\x01 \x01(\x05\x12*\n\x08response\x18\x02 \x01(\x0b\x32\x18.plugin.EndpointResponse\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"\x88\x03\n\x0f\x45ndpointRequest\x12\x37\n\x08metadata\x18\x01 \x03(\x0b\x32%.plugin.EndpointRequest.MetadataEntry\x12\x35\n\x07headers\x18\x02 \x03(\x0b\x32$.plugin.EndpointRequest.HeadersEntry\x12;\n\nparameters\x18\x03 \x03(\x0b\x32\'.plugin.EndpointRequest.ParametersEntry\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x42\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\x1a\x45\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\"\x8d\x03\n\x10\x45ndpointResponse\x12\x38\n\x08metadata\x18\x01 \x03(\x0b\x32&.plugin.EndpointResponse.MetadataEntry\x12\x36\n\x07headers\x18\x02 \x03(\x0b\x32%.plugin.EndpointResponse.HeadersEntry\x12<\n\nparameters\x18\x03 \x03(\x0b\x32(.plugin.EndpointResponse.ParametersEntry\x12\r\n\x05value\x18\x04 \x01(\x0c\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x42\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\x1a\x45\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\"\x1c\n\nStringList\x12\x0e\n\x06values\x18\x01 \x03(\t\"9\n\x0fRegisterRequest\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ult_secure\x18\x02 \x01(\x08\"=\n\x10RegisterResponse\x12)\n\tendpoints\x18\x01 \x03(\x0b\x32\x16.plugin.PluginEndpoint\"\x8c\x02\n\x0ePluginEndpoint\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0e\n\x06secure\x18\x04 \x01(\x08\x12\x12\n\nauth_group\x18\x05 \x01(\t\x12 \n\x18\x65xpected_metadata_schema\x18\x06 \x01(\t\x12\x1f\n\x17\x65xpected_headers_schema\x18\x07 \x01(\t\x12\"\n\x1a\x65xpected_parameters_schema\x18\x08 \x01(\t\x12\x1c\n\x14\x65xpected_body_schema\x18\t \x01(\t\x12\x1e\n\x16\x65xpected_output_schema\x18\n \x01(\t\"\r\n\x0bLoggingArgs\"&\n\x08LogField\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"`\n\nLogMessage\x12\x1f\n\x05level\x18\x01 \x01(\x0e\x32\x10.plugin.LogLevel\x12\x0f\n\x07message\x18\x02 \x01(\t\x12 \n\x06\x66ields\x18\x03 \x03(\x0b\x32\x10.plugin.LogField*B\n\x08LogLevel\x12\t\n\x05\x44\x45\x42UG\x10\x00\x12\x08\n\x04INFO\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\t\n\x05\x46\x41TAL\x10\x04\x32\xd3\x01\n\rPluginService\x12=\n\x08Register\x12\x17.plugin.RegisterRequest\x1a\x18.plugin.RegisterResponse\x12G\n\x04\x43\x61ll\x12\x1e.plugin.EndpointRequestMessage\x1a\x1f.plugin.EndpointResponseMessage\x12:\n\rLoggingStream\x12\x13.plugin.LoggingArgs\x1a\x12.plugin.LogMessage0\x01\x42KZIgithub.com/intel-innersource/frameworks.automation.dtac.agent/api/grpc/gob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'plugin_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZIgithub.com/intel-innersource/frameworks.automation.dtac.agent/api/grpc/go'
  _ENDPOINTREQUEST_METADATAENTRY._options = None
  _ENDPOINTREQUEST_METADATAENTRY._serialized_options = b'8\001'
  _ENDPOINTREQUEST_HEADERSENTRY._options = None
  _ENDPOINTREQUEST_HEADERSENTRY._serialized_options = b'8\001'
  _ENDPOINTREQUEST_PARAMETERSENTRY._options = None
  _ENDPOINTREQUEST_PARAMETERSENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_METADATAENTRY._options = None
  _ENDPOINTRESPONSE_METADATAENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_HEADERSENTRY._options = None
  _ENDPOINTRESPONSE_HEADERSENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_PARAMETERSENTRY._options = None
  _ENDPOINTRESPONSE_PARAMETERSENTRY._serialized_options = b'8\001'
  _globals['_LOGLEVEL']._serialized_start=1577
  _globals['_LOGLEVEL']._serialized_end=1643
  _globals['_ENDPOINTREQUESTMESSAGE']._serialized_start=24
  _globals['_ENDPOINTREQUESTMESSAGE']._serialized_end=106
  _globals['_ENDPOINTRESPONSEMESSAGE']._serialized_start=108
  _globals['_ENDPOINTRESPONSEMESSAGE']._serialized_end=204
  _globals['_ENDPOINTREQUEST']._serialized_start=207
  _globals['_ENDPOINTREQUEST']._serialized_end=599
  _globals['_ENDPOINTREQUEST_METADATAENTRY']._serialized_start=413
  _globals['_ENDPOINTREQUEST_METADATAENTRY']._serialized_end=460
  _globals['_ENDPOINTREQUEST_HEADERSENTRY']._serialized_start=462
  _globals['_ENDPOINTREQUEST_HEADERSENTRY']._serialized_end=528
  _globals['_ENDPOINTREQUEST_PARAMETERSENTRY']._serialized_start=530
  _globals['_ENDPOINTREQUEST_PARAMETERSENTRY']._serialized_end=599
  _globals['_ENDPOINTRESPONSE']._serialized_start=602
  _globals['_ENDPOINTRESPONSE']._serialized_end=999
  _globals['_ENDPOINTRESPONSE_METADATAENTRY']._serialized_start=413
  _globals['_ENDPOINTRESPONSE_METADATAENTRY']._serialized_end=460
  _globals['_ENDPOINTRESPONSE_HEADERSENTRY']._serialized_start=462
  _globals['_ENDPOINTRESPONSE_HEADERSENTRY']._serialized_end=528
  _globals['_ENDPOINTRESPONSE_PARAMETERSENTRY']._serialized_start=530
  _globals['_ENDPOINTRESPONSE_PARAMETERSENTRY']._serialized_end=599
  _globals['_STRINGLIST']._serialized_start=1001
  _globals['_STRINGLIST']._serialized_end=1029
  _globals['_REGISTERREQUEST']._serialized_start=1031
  _globals['_REGISTERREQUEST']._serialized_end=1088
  _globals['_REGISTERRESPONSE']._serialized_start=1090
  _globals['_REGISTERRESPONSE']._serialized_end=1151
  _globals['_PLUGINENDPOINT']._serialized_start=1154
  _globals['_PLUGINENDPOINT']._serialized_end=1422
  _globals['_LOGGINGARGS']._serialized_start=1424
  _globals['_LOGGINGARGS']._serialized_end=1437
  _globals['_LOGFIELD']._serialized_start=1439
  _globals['_LOGFIELD']._serialized_end=1477
  _globals['_LOGMESSAGE']._serialized_start=1479
  _globals['_LOGMESSAGE']._serialized_end=1575
  _globals['_PLUGINSERVICE']._serialized_start=1646
  _globals['_PLUGINSERVICE']._serialized_end=1857
# @@protoc_insertion_point(module_scope)
